from .apikey import ApiKeyProvider
from .authentication import (PublicProvider, RunClientWithCredentials,
                             new_client_with_credentials)
from .credentials import (Config, ContextConfig, Credentials, WorkspaceConfig,
                          load_credentials, load_credentials_from_settings)
from .device_mode import (BearerToken, DeviceLogin, DeviceLoginFinalizeRequest,
                          DeviceLoginFinalizeResponse, DeviceLoginResponse)

__all__ = (
    "ApiKeyProvider",
    "PublicProvider",
    "RunClientWithCredentials",
    "new_client_with_credentials",
    "Config",
    "ContextConfig",
    "Credentials",
    "WorkspaceConfig",
    "load_credentials",
    "load_credentials_from_settings",
    "BearerToken",
    "DeviceLogin",
    "DeviceLoginFinalizeRequest",
    "DeviceLoginFinalizeResponse",
    "DeviceLoginResponse"
)


from typing import List

from beamlit.api.models import list_models
from beamlit.authentication import (RunClientWithCredentials,
                                    load_credentials_from_settings,
                                    new_client_with_credentials)
from beamlit.common.settings import init, init_agent
from beamlit.models.model import Model
from beamlit.run import RunClient

settings = init()
credentials = load_credentials_from_settings(settings)

client_config = RunClientWithCredentials(
    credentials=credentials,
    workspace=settings.workspace,
)
client = new_client_with_credentials(client_config)
init_agent(client=client)
models: List[Model] = list_models.sync(client=client)
print(settings.agent_model)

run_client = RunClient(client=client)
response = run_client.run(
    "function",
    "math",
    settings.environment,
    method="POST",
    json={"query": "4+4"}
)

print(response.json())


from dataclasses import dataclass
from typing import Generator

from httpx import Auth, Request, Response

from ..client import AuthenticatedClient
from .apikey import ApiKeyProvider
from .clientcredentials import ClientCredentials
from .credentials import Credentials
from .device_mode import BearerToken


class PublicProvider(Auth):
    def auth_flow(self, request: Request) -> Generator[Request, Response, None]:
        yield request



@dataclass
class RunClientWithCredentials:
    credentials: Credentials
    workspace: str
    api_url: str = "https://api.beamlit.dev/v0"
    run_url: str = "https://run.beamlit.dev/v0"


def new_client_with_credentials(config: RunClientWithCredentials):
    provider: Auth = None
    if config.credentials.api_key:
        provider = ApiKeyProvider(config.credentials, config.workspace)
    elif config.credentials.access_token:
        provider = BearerToken(config.credentials, config.workspace, config.api_url)
    elif config.credentials.client_credentials:
        provider = ClientCredentials(config.credentials, config.workspace, config.api_url)
    else:
        provider = PublicProvider()

    return AuthenticatedClient(base_url=config.api_url, provider=provider)



from typing import Any, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ACL")


@_attrs_define
class ACL:
    """ACL

    Attributes:
        created_at (Union[Unset, str]): The date and time when the resource was created
        created_by (Union[Unset, str]): The user or service account who created the resource
        updated_at (Union[Unset, str]): The date and time when the resource was updated
        updated_by (Union[Unset, str]): The user or service account who updated the resource
        id (Union[Unset, str]): ACL id
        resource_id (Union[Unset, str]): Resource ID
        resource_type (Union[Unset, str]): Resource type
        role (Union[Unset, str]): Role
        subject_id (Union[Unset, str]): Subject ID
        subject_type (Union[Unset, str]): Subject type
        workspace (Union[Unset, str]): Workspace name
    """

    created_at: Union[Unset, str] = UNSET
    created_by: Union[Unset, str] = UNSET
    updated_at: Union[Unset, str] = UNSET
    updated_by: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    resource_id: Union[Unset, str] = UNSET
    resource_type: Union[Unset, str] = UNSET
    role: Union[Unset, str] = UNSET
    subject_id: Union[Unset, str] = UNSET
    subject_type: Union[Unset, str] = UNSET
    workspace: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        created_at = self.created_at

        created_by = self.created_by

        updated_at = self.updated_at

        updated_by = self.updated_by

        id = self.id

        resource_id = self.resource_id

        resource_type = self.resource_type

        role = self.role

        subject_id = self.subject_id

        subject_type = self.subject_type

        workspace = self.workspace

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if created_by is not UNSET:
            field_dict["created_by"] = created_by
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if updated_by is not UNSET:
            field_dict["updated_by"] = updated_by
        if id is not UNSET:
            field_dict["id"] = id
        if resource_id is not UNSET:
            field_dict["resource_id"] = resource_id
        if resource_type is not UNSET:
            field_dict["resource_type"] = resource_type
        if role is not UNSET:
            field_dict["role"] = role
        if subject_id is not UNSET:
            field_dict["subject_id"] = subject_id
        if subject_type is not UNSET:
            field_dict["subject_type"] = subject_type
        if workspace is not UNSET:
            field_dict["workspace"] = workspace

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        created_at = d.pop("created_at", UNSET)

        created_by = d.pop("created_by", UNSET)

        updated_at = d.pop("updated_at", UNSET)

        updated_by = d.pop("updated_by", UNSET)

        id = d.pop("id", UNSET)

        resource_id = d.pop("resource_id", UNSET)

        resource_type = d.pop("resource_type", UNSET)

        role = d.pop("role", UNSET)

        subject_id = d.pop("subject_id", UNSET)

        subject_type = d.pop("subject_type", UNSET)

        workspace = d.pop("workspace", UNSET)

        acl = cls(
            created_at=created_at,
            created_by=created_by,
            updated_at=updated_at,
            updated_by=updated_by,
            id=id,
            resource_id=resource_id,
            resource_type=resource_type,
            role=role,
            subject_id=subject_id,
            subject_type=subject_type,
            workspace=workspace,
        )

        acl.additional_properties = d
        return acl

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.model import Model
from ...models.model_with_deployments import ModelWithDeployments
from ...types import Response


def _get_kwargs(
    model_name: str,
    *,
    body: ModelWithDeployments,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "put",
        "url": f"/models/{model_name}",
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Model]:
    if response.status_code == 200:
        response_200 = Model.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Model]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    model_name: str,
    *,
    client: AuthenticatedClient,
    body: ModelWithDeployments,
) -> Response[Model]:
    """Create or update model

     Update a model by name.

    Args:
        model_name (str):
        body (ModelWithDeployments): Logical object representing a model but with deployment
            definition inside

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Model]
    """

    kwargs = _get_kwargs(
        model_name=model_name,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    model_name: str,
    *,
    client: AuthenticatedClient,
    body: ModelWithDeployments,
) -> Optional[Model]:
    """Create or update model

     Update a model by name.

    Args:
        model_name (str):
        body (ModelWithDeployments): Logical object representing a model but with deployment
            definition inside

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Model
    """

    return sync_detailed(
        model_name=model_name,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    model_name: str,
    *,
    client: AuthenticatedClient,
    body: ModelWithDeployments,
) -> Response[Model]:
    """Create or update model

     Update a model by name.

    Args:
        model_name (str):
        body (ModelWithDeployments): Logical object representing a model but with deployment
            definition inside

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Model]
    """

    kwargs = _get_kwargs(
        model_name=model_name,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    model_name: str,
    *,
    client: AuthenticatedClient,
    body: ModelWithDeployments,
) -> Optional[Model]:
    """Create or update model

     Update a model by name.

    Args:
        model_name (str):
        body (ModelWithDeployments): Logical object representing a model but with deployment
            definition inside

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Model
    """

    return (
        await asyncio_detailed(
            model_name=model_name,
            client=client,
            body=body,
        )
    ).parsed



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.function_deployment import FunctionDeployment
    from ..models.labels_type_0 import LabelsType0


T = TypeVar("T", bound="FunctionWithDeployments")


@_attrs_define
class FunctionWithDeployments:
    """Logical object representing a function but with deployment definition inside

    Attributes:
        created_at (Union[Unset, str]): The date and time when the resource was created
        created_by (Union[Unset, str]): The user or service account who created the resource
        updated_at (Union[Unset, str]): The date and time when the resource was updated
        updated_by (Union[Unset, str]): The user or service account who updated the resource
        display_name (Union[Unset, str]): Function display name
        labels (Union['LabelsType0', None, Unset]): Labels
        name (Union[Unset, str]): Function name
        workspace (Union[Unset, str]): Workspace name
        deployments (Union[Unset, List['FunctionDeployment']]):
    """

    created_at: Union[Unset, str] = UNSET
    created_by: Union[Unset, str] = UNSET
    updated_at: Union[Unset, str] = UNSET
    updated_by: Union[Unset, str] = UNSET
    display_name: Union[Unset, str] = UNSET
    labels: Union["LabelsType0", None, Unset] = UNSET
    name: Union[Unset, str] = UNSET
    workspace: Union[Unset, str] = UNSET
    deployments: Union[Unset, List["FunctionDeployment"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.labels_type_0 import LabelsType0

        created_at = self.created_at

        created_by = self.created_by

        updated_at = self.updated_at

        updated_by = self.updated_by

        display_name = self.display_name

        labels: Union[Dict[str, Any], None, Unset]
        if isinstance(self.labels, Unset):
            labels = UNSET
        elif isinstance(self.labels, LabelsType0):
            labels = self.labels.to_dict()
        else:
            labels = self.labels

        name = self.name

        workspace = self.workspace

        deployments: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.deployments, Unset):
            deployments = []
            for componentsschemas_function_deployments_item_data in self.deployments:
                componentsschemas_function_deployments_item = componentsschemas_function_deployments_item_data.to_dict()
                deployments.append(componentsschemas_function_deployments_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if created_by is not UNSET:
            field_dict["created_by"] = created_by
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if updated_by is not UNSET:
            field_dict["updated_by"] = updated_by
        if display_name is not UNSET:
            field_dict["display_name"] = display_name
        if labels is not UNSET:
            field_dict["labels"] = labels
        if name is not UNSET:
            field_dict["name"] = name
        if workspace is not UNSET:
            field_dict["workspace"] = workspace
        if deployments is not UNSET:
            field_dict["deployments"] = deployments

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.function_deployment import FunctionDeployment
        from ..models.labels_type_0 import LabelsType0

        d = src_dict.copy()
        created_at = d.pop("created_at", UNSET)

        created_by = d.pop("created_by", UNSET)

        updated_at = d.pop("updated_at", UNSET)

        updated_by = d.pop("updated_by", UNSET)

        display_name = d.pop("display_name", UNSET)

        def _parse_labels(data: object) -> Union["LabelsType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_labels_type_0 = LabelsType0.from_dict(data)

                return componentsschemas_labels_type_0
            except:  # noqa: E722
                pass
            return cast(Union["LabelsType0", None, Unset], data)

        labels = _parse_labels(d.pop("labels", UNSET))

        name = d.pop("name", UNSET)

        workspace = d.pop("workspace", UNSET)

        deployments = []
        _deployments = d.pop("deployments", UNSET)
        for componentsschemas_function_deployments_item_data in _deployments or []:
            componentsschemas_function_deployments_item = FunctionDeployment.from_dict(
                componentsschemas_function_deployments_item_data
            )

            deployments.append(componentsschemas_function_deployments_item)

        function_with_deployments = cls(
            created_at=created_at,
            created_by=created_by,
            updated_at=updated_at,
            updated_by=updated_by,
            display_name=display_name,
            labels=labels,
            name=name,
            workspace=workspace,
            deployments=deployments,
        )

        function_with_deployments.additional_properties = d
        return function_with_deployments

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import logging
import os


class ColoredFormatter(logging.Formatter):
    COLORS = {
        'DEBUG': '\033[1;36m',    # Cyan
        'INFO': '\033[1;32m',     # Green
        'WARNING': '\033[1;33m',  # Yellow
        'ERROR': '\033[1;31m',    # Red
        'CRITICAL': '\033[1;41m'  # Red background
    }

    def format(self, record):
        color = self.COLORS.get(record.levelname, '\033[0m')
        record.levelname = f"{color}{record.levelname}\033[0m"
        return super().format(record)

def init(log_level: str):
    logging.getLogger("uvicorn.access").handlers.clear()
    logging.getLogger("uvicorn.access").propagate = False
    logging.getLogger("httpx").handlers.clear()
    logging.getLogger("httpx").propagate = False

    handler = logging.StreamHandler()
    handler.setFormatter(ColoredFormatter('%(levelname)s:\t  %(name)s - %(message)s'))
    logging.basicConfig(
        level=log_level,
        handlers=[handler]
    )


from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.metric import Metric
    from ..models.qps import QPS
    from ..models.resource_deployment_metrics_inference_per_region_type_0 import (
        ResourceDeploymentMetricsInferencePerRegionType0,
    )
    from ..models.resource_deployment_metrics_inference_per_second_per_region_type_0 import (
        ResourceDeploymentMetricsInferencePerSecondPerRegionType0,
    )
    from ..models.resource_deployment_metrics_query_per_region_per_code_type_0 import (
        ResourceDeploymentMetricsQueryPerRegionPerCodeType0,
    )
    from ..models.resource_deployment_metrics_query_per_second_per_region_per_code_type_0 import (
        ResourceDeploymentMetricsQueryPerSecondPerRegionPerCodeType0,
    )


T = TypeVar("T", bound="ResourceDeploymentMetrics")


@_attrs_define
class ResourceDeploymentMetrics:
    """Metrics for a single resource deployment (eg. model deployment, function deployment)

    Attributes:
        inference_global (Union[Unset, List['Metric']]): Array of metrics
        inference_per_region (Union['ResourceDeploymentMetricsInferencePerRegionType0', None, Unset]): Historical
            requests (in last 24 hours) per location, for the model deployment
        inference_per_second_global (Union[Unset, List['Metric']]): Array of metrics
        inference_per_second_per_region (Union['ResourceDeploymentMetricsInferencePerSecondPerRegionType0', None,
            Unset]): Historical requests per second (RPS) per location, for the model deployment
        query_global (Union[Unset, float]): Number of requests done on the resource for the model deployment
        query_per_code_global (Union[Unset, QPS]): Query per second per element, can be per response status code (e.g.
            200, 400) or per location
        query_per_region (Union[Unset, QPS]): Query per second per element, can be per response status code (e.g. 200,
            400) or per location
        query_per_region_per_code (Union['ResourceDeploymentMetricsQueryPerRegionPerCodeType0', None, Unset]): Number of
            requests done on the resource for the model deployment
        query_per_second_global (Union[Unset, float]): RPS value (in last 24 hours) for the model deployment globally
        query_per_second_per_code_global (Union[Unset, QPS]): Query per second per element, can be per response status
            code (e.g. 200, 400) or per location
        query_per_second_per_region (Union[Unset, QPS]): Query per second per element, can be per response status code
            (e.g. 200, 400) or per location
        query_per_second_per_region_per_code (Union['ResourceDeploymentMetricsQueryPerSecondPerRegionPerCodeType0',
            None, Unset]): RPS value (in last 24 hours) per response status code per location, for the model deployment
    """

    inference_global: Union[Unset, List["Metric"]] = UNSET
    inference_per_region: Union["ResourceDeploymentMetricsInferencePerRegionType0", None, Unset] = UNSET
    inference_per_second_global: Union[Unset, List["Metric"]] = UNSET
    inference_per_second_per_region: Union["ResourceDeploymentMetricsInferencePerSecondPerRegionType0", None, Unset] = (
        UNSET
    )
    query_global: Union[Unset, float] = UNSET
    query_per_code_global: Union[Unset, "QPS"] = UNSET
    query_per_region: Union[Unset, "QPS"] = UNSET
    query_per_region_per_code: Union["ResourceDeploymentMetricsQueryPerRegionPerCodeType0", None, Unset] = UNSET
    query_per_second_global: Union[Unset, float] = UNSET
    query_per_second_per_code_global: Union[Unset, "QPS"] = UNSET
    query_per_second_per_region: Union[Unset, "QPS"] = UNSET
    query_per_second_per_region_per_code: Union[
        "ResourceDeploymentMetricsQueryPerSecondPerRegionPerCodeType0", None, Unset
    ] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.resource_deployment_metrics_inference_per_region_type_0 import (
            ResourceDeploymentMetricsInferencePerRegionType0,
        )
        from ..models.resource_deployment_metrics_inference_per_second_per_region_type_0 import (
            ResourceDeploymentMetricsInferencePerSecondPerRegionType0,
        )
        from ..models.resource_deployment_metrics_query_per_region_per_code_type_0 import (
            ResourceDeploymentMetricsQueryPerRegionPerCodeType0,
        )
        from ..models.resource_deployment_metrics_query_per_second_per_region_per_code_type_0 import (
            ResourceDeploymentMetricsQueryPerSecondPerRegionPerCodeType0,
        )

        inference_global: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.inference_global, Unset):
            inference_global = []
            for componentsschemas_array_metric_item_data in self.inference_global:
                componentsschemas_array_metric_item = componentsschemas_array_metric_item_data.to_dict()
                inference_global.append(componentsschemas_array_metric_item)

        inference_per_region: Union[Dict[str, Any], None, Unset]
        if isinstance(self.inference_per_region, Unset):
            inference_per_region = UNSET
        elif isinstance(self.inference_per_region, ResourceDeploymentMetricsInferencePerRegionType0):
            inference_per_region = self.inference_per_region.to_dict()
        else:
            inference_per_region = self.inference_per_region

        inference_per_second_global: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.inference_per_second_global, Unset):
            inference_per_second_global = []
            for componentsschemas_array_metric_item_data in self.inference_per_second_global:
                componentsschemas_array_metric_item = componentsschemas_array_metric_item_data.to_dict()
                inference_per_second_global.append(componentsschemas_array_metric_item)

        inference_per_second_per_region: Union[Dict[str, Any], None, Unset]
        if isinstance(self.inference_per_second_per_region, Unset):
            inference_per_second_per_region = UNSET
        elif isinstance(
            self.inference_per_second_per_region, ResourceDeploymentMetricsInferencePerSecondPerRegionType0
        ):
            inference_per_second_per_region = self.inference_per_second_per_region.to_dict()
        else:
            inference_per_second_per_region = self.inference_per_second_per_region

        query_global = self.query_global

        query_per_code_global: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.query_per_code_global, Unset):
            query_per_code_global = self.query_per_code_global.to_dict()

        query_per_region: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.query_per_region, Unset):
            query_per_region = self.query_per_region.to_dict()

        query_per_region_per_code: Union[Dict[str, Any], None, Unset]
        if isinstance(self.query_per_region_per_code, Unset):
            query_per_region_per_code = UNSET
        elif isinstance(self.query_per_region_per_code, ResourceDeploymentMetricsQueryPerRegionPerCodeType0):
            query_per_region_per_code = self.query_per_region_per_code.to_dict()
        else:
            query_per_region_per_code = self.query_per_region_per_code

        query_per_second_global = self.query_per_second_global

        query_per_second_per_code_global: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.query_per_second_per_code_global, Unset):
            query_per_second_per_code_global = self.query_per_second_per_code_global.to_dict()

        query_per_second_per_region: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.query_per_second_per_region, Unset):
            query_per_second_per_region = self.query_per_second_per_region.to_dict()

        query_per_second_per_region_per_code: Union[Dict[str, Any], None, Unset]
        if isinstance(self.query_per_second_per_region_per_code, Unset):
            query_per_second_per_region_per_code = UNSET
        elif isinstance(
            self.query_per_second_per_region_per_code, ResourceDeploymentMetricsQueryPerSecondPerRegionPerCodeType0
        ):
            query_per_second_per_region_per_code = self.query_per_second_per_region_per_code.to_dict()
        else:
            query_per_second_per_region_per_code = self.query_per_second_per_region_per_code

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if inference_global is not UNSET:
            field_dict["inference_global"] = inference_global
        if inference_per_region is not UNSET:
            field_dict["inference_per_region"] = inference_per_region
        if inference_per_second_global is not UNSET:
            field_dict["inference_per_second_global"] = inference_per_second_global
        if inference_per_second_per_region is not UNSET:
            field_dict["inference_per_second_per_region"] = inference_per_second_per_region
        if query_global is not UNSET:
            field_dict["query_global"] = query_global
        if query_per_code_global is not UNSET:
            field_dict["query_per_code_global"] = query_per_code_global
        if query_per_region is not UNSET:
            field_dict["query_per_region"] = query_per_region
        if query_per_region_per_code is not UNSET:
            field_dict["query_per_region_per_code"] = query_per_region_per_code
        if query_per_second_global is not UNSET:
            field_dict["query_per_second_global"] = query_per_second_global
        if query_per_second_per_code_global is not UNSET:
            field_dict["query_per_second_per_code_global"] = query_per_second_per_code_global
        if query_per_second_per_region is not UNSET:
            field_dict["query_per_second_per_region"] = query_per_second_per_region
        if query_per_second_per_region_per_code is not UNSET:
            field_dict["query_per_second_per_region_per_code"] = query_per_second_per_region_per_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.metric import Metric
        from ..models.qps import QPS
        from ..models.resource_deployment_metrics_inference_per_region_type_0 import (
            ResourceDeploymentMetricsInferencePerRegionType0,
        )
        from ..models.resource_deployment_metrics_inference_per_second_per_region_type_0 import (
            ResourceDeploymentMetricsInferencePerSecondPerRegionType0,
        )
        from ..models.resource_deployment_metrics_query_per_region_per_code_type_0 import (
            ResourceDeploymentMetricsQueryPerRegionPerCodeType0,
        )
        from ..models.resource_deployment_metrics_query_per_second_per_region_per_code_type_0 import (
            ResourceDeploymentMetricsQueryPerSecondPerRegionPerCodeType0,
        )

        d = src_dict.copy()
        inference_global = []
        _inference_global = d.pop("inference_global", UNSET)
        for componentsschemas_array_metric_item_data in _inference_global or []:
            componentsschemas_array_metric_item = Metric.from_dict(componentsschemas_array_metric_item_data)

            inference_global.append(componentsschemas_array_metric_item)

        def _parse_inference_per_region(
            data: object,
        ) -> Union["ResourceDeploymentMetricsInferencePerRegionType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                inference_per_region_type_0 = ResourceDeploymentMetricsInferencePerRegionType0.from_dict(data)

                return inference_per_region_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ResourceDeploymentMetricsInferencePerRegionType0", None, Unset], data)

        inference_per_region = _parse_inference_per_region(d.pop("inference_per_region", UNSET))

        inference_per_second_global = []
        _inference_per_second_global = d.pop("inference_per_second_global", UNSET)
        for componentsschemas_array_metric_item_data in _inference_per_second_global or []:
            componentsschemas_array_metric_item = Metric.from_dict(componentsschemas_array_metric_item_data)

            inference_per_second_global.append(componentsschemas_array_metric_item)

        def _parse_inference_per_second_per_region(
            data: object,
        ) -> Union["ResourceDeploymentMetricsInferencePerSecondPerRegionType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                inference_per_second_per_region_type_0 = (
                    ResourceDeploymentMetricsInferencePerSecondPerRegionType0.from_dict(data)
                )

                return inference_per_second_per_region_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ResourceDeploymentMetricsInferencePerSecondPerRegionType0", None, Unset], data)

        inference_per_second_per_region = _parse_inference_per_second_per_region(
            d.pop("inference_per_second_per_region", UNSET)
        )

        query_global = d.pop("query_global", UNSET)

        _query_per_code_global = d.pop("query_per_code_global", UNSET)
        query_per_code_global: Union[Unset, QPS]
        if isinstance(_query_per_code_global, Unset):
            query_per_code_global = UNSET
        else:
            query_per_code_global = QPS.from_dict(_query_per_code_global)

        _query_per_region = d.pop("query_per_region", UNSET)
        query_per_region: Union[Unset, QPS]
        if isinstance(_query_per_region, Unset):
            query_per_region = UNSET
        else:
            query_per_region = QPS.from_dict(_query_per_region)

        def _parse_query_per_region_per_code(
            data: object,
        ) -> Union["ResourceDeploymentMetricsQueryPerRegionPerCodeType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                query_per_region_per_code_type_0 = ResourceDeploymentMetricsQueryPerRegionPerCodeType0.from_dict(data)

                return query_per_region_per_code_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ResourceDeploymentMetricsQueryPerRegionPerCodeType0", None, Unset], data)

        query_per_region_per_code = _parse_query_per_region_per_code(d.pop("query_per_region_per_code", UNSET))

        query_per_second_global = d.pop("query_per_second_global", UNSET)

        _query_per_second_per_code_global = d.pop("query_per_second_per_code_global", UNSET)
        query_per_second_per_code_global: Union[Unset, QPS]
        if isinstance(_query_per_second_per_code_global, Unset):
            query_per_second_per_code_global = UNSET
        else:
            query_per_second_per_code_global = QPS.from_dict(_query_per_second_per_code_global)

        _query_per_second_per_region = d.pop("query_per_second_per_region", UNSET)
        query_per_second_per_region: Union[Unset, QPS]
        if isinstance(_query_per_second_per_region, Unset):
            query_per_second_per_region = UNSET
        else:
            query_per_second_per_region = QPS.from_dict(_query_per_second_per_region)

        def _parse_query_per_second_per_region_per_code(
            data: object,
        ) -> Union["ResourceDeploymentMetricsQueryPerSecondPerRegionPerCodeType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                query_per_second_per_region_per_code_type_0 = (
                    ResourceDeploymentMetricsQueryPerSecondPerRegionPerCodeType0.from_dict(data)
                )

                return query_per_second_per_region_per_code_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ResourceDeploymentMetricsQueryPerSecondPerRegionPerCodeType0", None, Unset], data)

        query_per_second_per_region_per_code = _parse_query_per_second_per_region_per_code(
            d.pop("query_per_second_per_region_per_code", UNSET)
        )

        resource_deployment_metrics = cls(
            inference_global=inference_global,
            inference_per_region=inference_per_region,
            inference_per_second_global=inference_per_second_global,
            inference_per_second_per_region=inference_per_second_per_region,
            query_global=query_global,
            query_per_code_global=query_per_code_global,
            query_per_region=query_per_region,
            query_per_region_per_code=query_per_region_per_code,
            query_per_second_global=query_per_second_global,
            query_per_second_per_code_global=query_per_second_per_code_global,
            query_per_second_per_region=query_per_second_per_region,
            query_per_second_per_region_per_code=query_per_second_per_region_per_code,
        )

        resource_deployment_metrics.additional_properties = d
        return resource_deployment_metrics

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
